﻿using FastCGI;
using Owin;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Demo.Owin.Hosting
{
    class WebServer : IDisposable
    {
        private readonly Thread listener;
        private readonly AppBuilder appBuilder;

        public WebServer(Type type)
        {
            var configurationMethod = type.GetMethod("Configuration", new Type[] { typeof(IAppBuilder) });

            if (configurationMethod == null)
            {
                throw new ArgumentException("Cannot find Configuration method");
            }

            var obj = Activator.CreateInstance(type);
            this.appBuilder = new AppBuilder();
            configurationMethod.Invoke(obj, new object[] { appBuilder });

            this.listener = new Thread(this.Start)
            {
                IsBackground = true
            };
            this.listener.Start();
        }

        public void Dispose()
        {
            // TODO: call Server.Stop(...)
        }

        private void Start()
        {
            var config = new Options();
            config.Bind = BindMode.CreateSocket;
            var addr = IPAddress.Parse("127.0.0.1");
            config.EndPoint = new IPEndPoint(addr, 9000);
            config.OnError = Console.WriteLine;
            Server.Start(this.HandleRequest, config);
        }

        private void HandleRequest(Request req, Response res)
        {
            var outputBuff = new byte[1000];

            // TODO: use middleware chaining instead a loop

            foreach (var appFunc in this.appBuilder.RegisteredMiddlewareObjects)
            {
                using (var ms = new MemoryStream(outputBuff))
                {
                    this.appBuilder.Properties["owin.RequestPath"] = req.ScriptName.Value;
                    this.appBuilder.Properties["owin.RequestQueryString"] = req.QueryString.Value;

                    this.appBuilder.Properties["owin.ResponseBody"] = ms;
                    this.appBuilder.Properties["owin.ResponseStatusCode"] = 0;

                    var task = appFunc(this.appBuilder.Properties);

                    // TODO: don't task.Wait() and use res.AsyncPut(outputBuff);

                    task.Wait();
                    res.Put(outputBuff);
                }
            }
        }
    }
}
